package uk.ac.cardiff.booking.api.tutor;

import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

import javax.transaction.Transactional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import lombok.extern.slf4j.Slf4j;
import uk.ac.cardiff.booking.api.module.ModuleRepository;
import uk.ac.cardiff.booking.api.transcripts.Transcript;
import uk.ac.cardiff.booking.api.transcripts.TranscriptsRepository;
import uk.ac.cardiff.booking.api.module.Module;

@Service
@Slf4j
public class AppointmentService {
  
  @Autowired
  private AppointmentRepository repo;

  @Autowired
  private TranscriptsRepository transcriptRepo;

  @Autowired
  private TutorRepository tutorRepo;

  @Autowired
  private ModuleRepository moduleRepo;

  @Transactional
  public List<AppointmentDto> getAppointments(Integer tutorId) {
    List<Appointment> appointments = repo.findByTutorId(tutorId);
    log.info("Retrieved appointments: ", appointments.toString());
    return appointments.stream()
      .map(appointment -> AppointmentDto.builder()
          .start(appointment.getAppointmentStart())
          .end(appointment.getAppointmentEnd())
          .title(appointment.getTitle())
          .authCode(appointment.getAuthCode())
          .id(appointment.getId())
          .transcriptName(appointment.getTranscript().getFilename())
          .transcriptId(appointment.getTranscript().getId())
          .build())
      .collect(Collectors.toList()); 
  }

  @Transactional
  public List<AppointmentDto> getAppointmentsByStudentId(String studentNumber, UUID authCode) {
    List<Appointment> appointments = repo.findByStudentNumberAndAuthCode(studentNumber, authCode);
    return appointments.stream()
      .map(appointment -> AppointmentDto.builder()
          .start(appointment.getAppointmentStart())
          .end(appointment.getAppointmentEnd())
          .title(appointment.getTitle())
          .authCode(appointment.getAuthCode())
          .id(appointment.getId())
          .tutorId(appointment.getTutor().getId())
          .tutor(appointment.getTutor().getName())
          .moduleCode(appointment.getModule().getModuleCode())
          .transcriptId(appointment.getTranscript().getId())
          .transcriptName(appointment.getTranscript().getFilename())
          .build())
      .collect(Collectors.toList()); 
  }

  @Transactional
  public AppointmentDto createAppointment(Integer tutorId, AppointmentDto newDto) {
    Tutor tutor = tutorRepo.findById(tutorId).orElseThrow(() -> new MissingTutorException());
    Module module = moduleRepo.findByModuleCode(newDto.getModuleCode());
    Transcript transcript = transcriptRepo.findById(newDto.getTranscriptId()).get();
    if (module == null) {
      throw new RuntimeException("Module not found");
    }
    Appointment appointment = new Appointment();
    appointment.setAppointmentEnd(newDto.getEnd());
    appointment.setAppointmentStart(newDto.getStart());
    appointment.setTitle(newDto.getTitle());
    appointment.setAuthCode(newDto.getAuthCode());
    appointment.setStudentNumber(newDto.getStudentNumber());

    appointment.setTranscript(transcript);
    appointment.setTutor(tutor);
    appointment.setModule(module);
    Appointment savedAppointment = repo.save(appointment);
    AppointmentDto dto = AppointmentDto.builder()
      .id(savedAppointment.getId())
      .start(newDto.getStart())
      .end(newDto.getEnd())
      .title(newDto.getTitle())
      .transcriptName(transcript.getFilename())
      .transcriptId(transcript.getId())
      .build();
    return dto;
  }

  @Transactional
  public void deleteAppointment(Integer tutorId, Integer appointmentId) {
    repo.deleteById(appointmentId);
  }
}